#!/bin/bash
# vim: set ft=sh sw=2 ts=2 :

#--------------------   LITTLE ENDIANS   -------------------------------
#--- This Version is without byte swap for alpha, absoft, intel & pgi --
# 
#-----------------------------------------------------------------------
#      SHELL-SCRIPT FOR DEFINING FREQUENTLY USED COMPILER OPTIONS
# 
#                     Written by Michael Beck (12/98)
#
# The following environment variables are defined in dependence of the
# current machine:
#    $FC:               Fortran compiler.
#    $CC:               C compiler.
#    $AR:               Archive command.
#    $QUANTICS_FFLAGS_OPT: Fortran flags for optimising the code.
#    $QUANTICS_FFLAGS_DEB: Fortran flags for debugging the code.
#    $QUANTICS_FFLAGS_INC: Fortran flags specifying path to search for
#                       include files.
#    $QUANTICS_FFLAGS_MOD: Fortran flags for where to put .mod files.
#    $EXTERNAL_BLAS:    Linker options that specify where to find an
#                       external BLAS library (see compile -x).
#    $EXTERNAL_LAPACK:  Linker options that specify where to find an
#                       external LAPACK library (see compile -x).
#    $QUANTICS_ADD_LIBS:   List of additional libraries to be linked.
#    $QUANTICS_CFLAGS:     Flags for C compiler.
#    $QUANTICS_ARFLAGS:    Flags for archive command "ar".
#    $QUANTICS_OP_SYSTEM:  Operating system.
#    $QUANTICS_MAKE_CMD:   "Make" command to be used.
#    $QUANTICS_FLUSH_EXT:  Selects the version of the "flush" command to be
#                       used.
#    $QUANTICS_F2C_STYLE:  Selects how Fortran symbols are seen from C:
#             value  |  Fortran symbol  |  C symbol
#           ---------+------------------+------------
#                 0  |  FOO             |  foo_
#                 1  |  FOO             |  foo
#                 2  |  FOO             |  FOO
#
# Must add explicit pass through cpp to treat f90 files (NOT DONE FOR ALL)
#-----------------------------------------------------------------------

      # (4.0 will compile but not run due to deficiencies in the gfortran I/O library)
      # for processor-specific optimizations, add to QUANTICS_FFLAGS_OPT:
      #   "-march=pentium4 -mfpmath=sse" (for Pentium IV)
      #   "-march=k8 -mfpmath=sse" (for AMD64)
      #
      # (8.1 and 8.2 will compile but require -fno-frontend-optimize due to a bug 
      #  in handling nested implied do loops)
      # "-march=native" should only be added to FFLAGS_OPT for code to be used
      # on a single cpu architecture.

if [ "$QUANTICS_PLATFORM" = "i386" ]; then
   PLATFORM="i686"
else
   PLATFORM=$QUANTICS_PLATFORM
fi

QUANTICS_OP_SYSTEM=`uname -s`
if [ "$QUANTICS_OP_SYSTEM" = "Darwin" ]; then
   MACPATH=`xcrun --show-sdk-path`
fi

QUANTICS_ARFLAGS=rlc  # Use U option for ar if possible
                      # for modern Linux systems where 'deterministic' ar 
                      # is the default
ar 2>&1 | fgrep ' [U] ' 2>&1 >/dev/null && QUANTICS_ARFLAGS=rlcU

if [ -z "$QUANTICS_GCC" ]; then QUANTICS_GCC=gcc; fi

#-----------------------------------------------------------------------
# SET COMPILER DEPENDENT OPTIONS
#-----------------------------------------------------------------------
case $QUANTICS_COMPILER in
  gfortran)
# remove last digit from version
    GCC_VERSION1=${GCC_VERSION%.*}
      # Linux linker, make, ..., with GNU Fortran/C compiler, version 4.1.1 or later, 
      # 32 or 64bit depending on platform.
     #
    if [ -z "$QUANTICS_GFORTRAN" ]; then
      echo "No gfortran version found." >&2
      exit 1
    fi
    if [ "$QUANTICS_COMPILE_PARA" = "mpi"  -o "$QUANTICS_COMPILE_PARA" = "mix" ]; then
       export FC="mpifort"
    else
       export FC="$QUANTICS_GFORTRAN"
    fi
    export CC="$QUANTICS_GCC"
    export AR=ar

# original flags
    QUANTICS_FFLAGS_DEB="-g -O0 -ffixed-line-length-none -frecord-marker=4 -Wall -Wsurprising -fbounds-check -DDEBUG"
    QUANTICS_FFLAGS_OPT="-O2 -ffixed-line-length-none -frecord-marker=4 -funroll-loops -fomit-frame-pointer "
    QUANTICS_CFLAGS="-O2"

# add machine-specific flags here, e.g. "-march=native" to optimize for the compiling host
    #QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -march=native"

# version dependent flags
    if [ "${GCC_VERSION1}" = "8.1" ]; then
       QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -fno-frontend-optimize -ffpe-summary=none"
    elif [ "${GCC_VERSION1}" = "8.2" ]; then
       QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -fno-frontend-optimize -ffpe-summary=none"
    elif [ "${GCC_VERSION1%.*}" -gt 6 ]; then
       QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -ffpe-summary=none"
    fi

# select parallelisation flags
    if [ "$QUANTICS_COMPILE_PARA" = "omp" ]; then
      QUANTICS_OMP_FLAGS="-DOMP -fopenmp"
    elif [ "$QUANTICS_COMPILE_PARA" = "mix" ]; then
      QUANTICS_OMP_FLAGS="-DOMP -fopenmp"
      QUANTICS_MPI_FLAGS="-DMPI"
    elif [ "$QUANTICS_COMPILE_PARA" = "mpi" ]; then
      QUANTICS_MPI_FLAGS="-DMPI"
    else
      QUANTICS_OMP_FLAGS=" "
      QUANTICS_MPI_FLAGS=" "
    fi

# select endianness
    if [ "$QUANTICS_COMPILE_END" = "little" ]; then
      QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -fconvert=little-endian"
      QUANTICS_FFLAGS_DEB=${QUANTICS_FFLAGS_DEB}" -fconvert=little-endian"
    elif [ "$QUANTICS_COMPILE_END" = "big" ]; then
      QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -fconvert=big-endian"
      QUANTICS_FFLAGS_DEB=${QUANTICS_FFLAGS_DEB}" -fconvert=big-endian"
    fi
    
# Horrible hack for gfortran-10
    GFORTRAN_VER="$(gfortran -dumpversion)"
    if [ "$(printf '%s\n' "10" "$GFORTRAN_VER" | sort -V | head -n1)" = "10" ]; then 
        QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -fallow-argument-mismatch"
        QUANTICS_FFLAGS_DEB=${QUANTICS_FFLAGS_DEB}" -fallow-argument-mismatch"    
    fi 



    QUANTICS_FFLAGS_INC="-I"
    QUANTICS_FFLAGS_MOD="-J"
    EXTERNAL_BLAS=-lblas
    EXTERNAL_LAPACK=-llapack
    QUANTICS_SCALAPACK_FLAGS="-DSCALAPACK"
    QUANTICS_SCALAPACK_LIBS="-lscalapack" # -lblacs or -L/path/to/scalapack..?
    if [ $QUANTICS_OP_SYSTEM = "Darwin" ]; then
       QUANTICS_ADD_LIBS="-lgfortran"
       if [ -n "$MACPATH" ]; then
          QUANTICS_ADD_LIBS="-lgfortran -isysroot $MACPATH"
          QUANTICS_CFLAGS=${QUANTICS_CFLAGS}" -I$MACPATH/usr/include"
       fi
    else
       QUANTICS_ADD_LIBS=" -lgfortran"
    fi
    QUANTICS_MAKE_CMD=make
    QUANTICS_FLUSH_EXT=_lib
    QUANTICS_F2C_STYLE=0
    ;;
  gfortran-4.9)
      # Linux linker, make, ..., with GNU Fortran/C compiler, version 4.9 or later, 
      # 32 or 64bit depending on platform.
     #
    if [ -z "$QUANTICS_GFORTRAN" ]; then
      echo "No gfortran version found." >&2
      exit 1
    fi
    if [ "$QUANTICS_COMPILE_PARA" = "mpi" -o "$QUANTICS_COMPILE_PARA" = "mix" ]; then
       export FC="mpifort"
    else
       export FC="$QUANTICS_GFORTRAN"
    fi
    export CC="$QUANTICS_GCC"
    export AR=ar
#original flags
    QUANTICS_FFLAGS_DEB="-g -O0 -ffixed-line-length-none -frecord-marker=4 -Wall -Wsurprising -fbounds-check -DDEBUG"
    QUANTICS_FFLAGS_OPT="-O2 -ffixed-line-length-none -frecord-marker=4 -march=native -funroll-loops -fomit-frame-pointer -ffpe-summary=none"
    QUANTICS_CFLAGS="-O2"
# add machine-specific flags here, e.g. "-march=native" to optimize for the compiling host
    #QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -march=native"
    QUANTICS_OMP_FLAGS="-DOMP -fopenmp"
    QUANTICS_MPI_FLAGS="-DMPI"
# select endianness
    if [ "$QUANTICS_COMPILE_END" = "little" ]; then
      QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -fconvert=little-endian"
      QUANTICS_FFLAGS_DEB=${QUANTICS_FFLAGS_DEB}" -fconvert=little-endian"
    elif [ "$QUANTICS_COMPILE_END" = "big" ]; then
      QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -fconvert=big-endian"
      QUANTICS_FFLAGS_DEB=${QUANTICS_FFLAGS_DEB}" -fconvert=big-endian"
    fi
    QUANTICS_FFLAGS_INC="-I"
    QUANTICS_FFLAGS_MOD="-J"
    EXTERNAL_BLAS=-lblas
    EXTERNAL_LAPACK=-llapack
    QUANTICS_SCALAPACK_FLAGS="-DSCALAPACK"
    QUANTICS_SCALAPACK_LIBS="-lscalapack" # -lblacs or -L/path/to/scalapack..?
    if [ $QUANTICS_OP_SYSTEM = "Darwin" ]; then
       QUANTICS_ADD_LIBS="-lgfortran"
       if [ -n "$MACPATH" ]; then
          QUANTICS_ADD_LIBS="-lgfortran -isysroot $MACPATH"
          QUANTICS_CFLAGS=${QUANTICS_CFLAGS}" -I$MACPATH/usr/include"
       fi
    else
       QUANTICS_ADD_LIBS=" -lgfortran"
    fi
    QUANTICS_MAKE_CMD=make
    QUANTICS_FLUSH_EXT=_lib
    QUANTICS_F2C_STYLE=0
    ;;
  gfortran-32)
      # Linux linker, make, ..., with GNU Fortran/C compiler, version 4.1.1 or later, 32bit
      # (4.0 will compile but not run due to deficiencies in the gfortran I/O library)
     #
    # Endianess can be overridden by the environment variable GFORTRAN_CONVERT_UNIT
    # ("native", "big_endian", "little_endian")
    if [ -z "$QUANTICS_GFORTRAN" ]; then
      echo "No gfortran version found." >&2
      exit 1
    fi
    if [ "$QUANTICS_COMPILE_PARA" = "mpi" ]; then
       export FC="mpifort"
    else
       export FC="$QUANTICS_GFORTRAN"
    fi
    export CC="$QUANTICS_GCC"
    export AR=ar
    QUANTICS_FFLAGS_DEB="-g -O -fconvert=little-endian -frecord-marker=4 -Wall -Wsurprising -DDEBUG "
    QUANTICS_FFLAGS_OPT="-O2 -fconvert=little-endian -frecord-marker=4 -ffast-math -funroll-loops -fomit-frame-pointer "
    QUANTICS_CFLAGS="-O2"
    # For better performance, change "-msse2" in the following to "-march=native".
    QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -msse2 -mfpmath=sse"
    if [ "$QUANTICS_PLATFORM" = "x86_64" ]; then
      QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -m32"
      QUANTICS_FFLAGS_DEB=${QUANTICS_FFLAGS_DEB}" -m32"
      QUANTICS_CFLAGS=${QUANTICS_CFLAGS}" -m32"
    fi
    QUANTICS_OMP_FLAGS="-DOMP -fopenmp"
    QUANTICS_MPI_FLAGS="-DMPI"
    QUANTICS_FFLAGS_INC="-I"
    QUANTICS_FFLAGS_MOD="-J"
    EXTERNAL_BLAS=-lblas
    EXTERNAL_LAPACK=-llapack
    if [ $QUANTICS_OP_SYSTEM = "Darwin" ]; then
       QUANTICS_ADD_LIBS="-lgfortran"
       if [ -n "$MACPATH" ]; then
          QUANTICS_ADD_LIBS="-lgfortran -isysroot $MACPATH"
          QUANTICS_CFLAGS=${QUANTICS_CFLAGS}" -I$MACPATH/usr/include"
       fi
    else
       QUANTICS_ADD_LIBS=" -lgfortran"
    fi
    QUANTICS_MAKE_CMD=make
    QUANTICS_FLUSH_EXT=_lib
    QUANTICS_F2C_STYLE=0
    ;;
  g95) # Linux linker, make, ..., with GNU C compiler and G95 Fortran compiler
       # (see http://www.g95.org)
    if [ "$QUANTICS_COMPILE_PARA" = "mpi" ]; then
       export FC="mpifort"
    else
       export FC=g95
    fi
    export CC=gcc
    export AR=ar
    # G95 builds on GCC so it accepts most of its options
    QUANTICS_FFLAGS_DEB="-g -O -fendian=little -fbounds-check -Wall -Wsurprising -DDEBUG "
    QUANTICS_FFLAGS_OPT="-O2 -fendian=little -ffast-math -funroll-loops -fomit-frame-pointer"
    QUANTICS_CFLAGS="-O2"
    # For better performance, change "-msse2" in the following to "-march=native".
    QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -msse2 -mfpmath=sse"
    QUANTICS_OMP_FLAGS="-DOMP -fopenmp"
    QUANTICS_MPI_FLAGS="-DMPI"
    QUANTICS_FFLAGS_INC="-I"
    QUANTICS_FFLAGS_MOD="-fmod="
    EXTERNAL_BLAS=-lblas
    EXTERNAL_LAPACK=-llapack
    QUANTICS_ADD_LIBS=
    QUANTICS_MAKE_CMD=make
    QUANTICS_FLUSH_EXT=
    QUANTICS_F2C_STYLE=0
    ;;
  intel7)  # Intel compilers for Linux (version 7.0)
    export FC=ifc
    export CC=gcc
    export AR=ar
    QUANTICS_FFLAGS_OPT="-O2 -Vaxlib -posixlib -mp -cm -complex_limited_range -ip -unroll -w"
    QUANTICS_FFLAGS_DEB="-g -Vaxlib -posixlib -mp -w -DDEBUG"
    QUANTICS_OMP_FLAGS="-DOMP -openmp"
    QUANTICS_FFLAGS_INC="-I"
    QUANTICS_FFLAGS_MOD="-module "
    # The following probably needs adjusting. If you have MKL, specify it here.
    EXTERNAL_BLAS=" "
    EXTERNAL_LAPACK=" "
    QUANTICS_ADD_LIBS=
    QUANTICS_CFLAGS="-O2"
    QUANTICS_MAKE_CMD=make
    QUANTICS_FLUSH_EXT=
    QUANTICS_F2C_STYLE=0
    ;;
  intel)  # Intel compilers for Linux
    if [ "$QUANTICS_COMPILE_PARA" = "mpi"  -o "$QUANTICS_COMPILE_PARA" = "mix" ]; then
       export FC="mpif90 -fc=ifort"
    else
       export FC=ifort
    fi
    export CC=icc
    export AR=ar
    # Change -x and -ax optimization flags depending on CPU
    QUANTICS_FFLAGS_OPT="-O2 -align all -warn nousage -xHost -ftz -ip -unroll -I${MKLROOT}/include"
    #QUANTICS_FFLAGS_OPT="-O2 -align all -warn none -msse4.2 -axAVX,CORE-AVX2,CORE-AVX512 -ftz -ip -unroll -I${MKLROOT}/include"
    QUANTICS_FFLAGS_DEB="-g -check all -traceback -fpe0 -DDEBUG"
    QUANTICS_FFLAGS_INC="-I"
    QUANTICS_FFLAGS_MOD="-module "
    QUANTICS_FFLAGS_0="-O0"
    # The following probably needs adjusting. If you have MKL, specify it here.
    if [ "$QUANTICS_COMPILE_PARA" = "omp" -o "$QUANTICS_COMPILE_PARA" = "mix" ]; then
      QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -qopenmp"
      QUANTICS_FFLAGS_DEB=${QUANTICS_FFLAGS_DEB}" -qopenmp"
      EXTERNAL_BLAS="-L${MKLROOT}/lib/intel64 -lmkl_intel_lp64 -lmkl_intel_thread -lmkl_core -liomp5 -lpthread -lm -ldl"
      EXTERNAL_LAPACK="-L${MKLROOT}/lib/intel64 -lmkl_intel_lp64 -lmkl_intel_thread -lmkl_core -liomp5 -lpthread -lm -ldl"
    else
      EXTERNAL_BLAS="-L${MKLROOT}/lib/intel64 -lmkl_intel_lp64 -lmkl_sequential -lmkl_core"
      EXTERNAL_LAPACK="-L${MKLROOT}/lib/intel64 -lmkl_intel_lp64 -lmkl_sequential -lmkl_core"
    fi
    QUANTICS_OMP_FLAGS="-DOMP -qopenmp"
    QUANTICS_MPI_FLAGS="-DMPI"
    QUANTICS_ADD_LIBS=
    QUANTICS_CFLAGS="-O2"
    QUANTICS_MAKE_CMD=make
    QUANTICS_FLUSH_EXT=
    QUANTICS_F2C_STYLE=0
    ;;
  pgf90)  # Linux linker, make, ..., with Portland-Group F90/CC compiler (v5.2)
          # pgcc does not work (?) so use gcc
    if [ "$QUANTICS_COMPILE_PARA" = "mpi" ]; then
       export FC="mpif90"
    else
       export FC=pgf90
    fi
#    export CC=pgcc
    export CC=gcc
    export AR=ar
    QUANTICS_FFLAGS_OPT="-O2 -fastsse -silent -mp"
    # processor-specific optimizations for Opteron:
#    if [ "$QUANTICS_PLATFORM" = "x86_64" ]; then
#      QUANTICS_FFLAGS_OPT=${QUANTICS_FFLAGS_OPT}" -tp=k8-64"
#    fi
    QUANTICS_FFLAGS_DEB="-g -C -Kieee -Ktrap=fp -Minform=warn -DDEBUG -mp"
    QUANTICS_FFLAGS_INC="-I"
    QUANTICS_FFLAGS_MOD="-module "
    QUANTICS_ADD_LIBS=
#    QUANTICS_CFLAGS="-fastsse -silent"
    QUANTICS_CFLAGS="  "
    QUANTICS_MAKE_CMD=make
    QUANTICS_FLUSH_EXT=_lib
    QUANTICS_F2C_STYLE=0
    ;;
  sunf90)  #   sunf90)  # Sun compiler - unsure if le or be
    export FC=sunf90
    export CC=suncc
    export AR=ar
    QUANTICS_FFLAGS_OPT="-O2"
    QUANTICS_FFLAGS_DEB="-g"
    QUANTICS_FFLAGS_INC="-I"
    QUANTICS_FFLAGS_MOD="-M" # this is probably wrong
    QUANTICS_ADD_LIBS=
    QUANTICS_CFLAGS=" "
    QUANTICS_MAKE_CMD=make
    QUANTICS_FLUSH_EXT=_lib
    QUANTICS_F2C_STYLE=0
    ;;
  pwr5_omp)   # IBM compiler, linker, ..., with POWER5 optimisation (up to 2GB RAM usage) + OpenMP
    export FC=xlf95_r
    export CC=cc
    export AR=ar
    QUANTICS_FFLAGS_OPT="-qsmp=omp -qnosave -O3 -qstrict -Q -qarch=auto -qtune=auto -bmaxdata:0x80000000"
    QUANTICS_FFLAGS_DEB="-g -C -d -qfullpath -qflttrap=overflow:zerodivide:enable "
    QUANTICS_FFLAGS_INC="-WF,-I" # or just "-I" according to man page
    QUANTICS_FFLAGS_MOD="-qmoddir="
    QUANTICS_ADD_LIBS=
    QUANTICS_CFLAGS="-O2 -bmaxdata:0x80000000"
    QUANTICS_MAKE_CMD=make
    QUANTICS_FLUSH_EXT=
    QUANTICS_F2C_STYLE=1
#   For 64 bit usage, incomment the following 3 lines.
    export OBJECT_MODE=64
    QUANTICS_FFLAGS_OPT="$QUANTICS_FFLAGS_OPT -q64"
    QUANTICS_FFLAGS_DEB="$QUANTICS_FFLAGS_DEB -q64"
    ;;
  default)  # Rest of the world
    export FC=f90
    export CC=cc
    export AR=ar
    QUANTICS_FFLAGS_OPT="-O"
    QUANTICS_FFLAGS_DEB="-g -DDEBUG"
    QUANTICS_FFLAGS_0="-O0"
    QUANTICS_FFLAGS_INC="-I"
    QUANTICS_FFLAGS_INC="-J"
    QUANTICS_ADD_LIBS=
    QUANTICS_CFLAGS="-O"
    QUANTICS_OP_SYSTEM=UNIX
    QUANTICS_MAKE_CMD=make
    QUANTICS_FLUSH_EXT=
    QUANTICS_F2C_STYLE=0
    ;;
  *)  # Unknown compiler
    echo '### Unknown compiler: '$QUANTICS_COMPILER
    exit 1  
    ;;
esac

#-----------------------------------------------------------------------------
# Let QUANTICS_FFLAGS_0 contain the options which are common in QUANTICS_FFLAGS_DEB
# and QUANTICS_FFLAGS_OPT, if previously unset.
#-----------------------------------------------------------------------------
if [ -z "$QUANTICS_FFLAGS_0" ]; then
   mfopt=$QUANTICS_FFLAGS_OPT
   for i in $QUANTICS_FFLAGS_DEB ; do
      ii=`echo $mfopt | fgrep -w -- $i`
      if [ "$i" = "-O0" ]; then ii=O0; fi
      if [ -n "$ii" ]; then
         QUANTICS_FFLAGS_0=$QUANTICS_FFLAGS_0' '$i    # Store  the found item
         mfopt=`echo $mfopt | sed -e "s/$i//"`  # Remove the found item
      fi
   done
fi

